@@ -4,25 +4,33 @@ from .bsblan import BSBLAN, BSBLANConfig
 from .exceptions import BSBLANAuthError, BSBLANConnectionError, BSBLANError
 from .models import (
     Device,
+    DeviceTime,
     DHWTimeSwitchPrograms,
+    HotWaterConfig,
+    HotWaterSchedule,
     HotWaterState,
     Info,
     Sensor,
+    SetHotWaterParam,
     State,
     StaticState,
 )
 
 __all__ = [
     "BSBLAN",
-    "BSBLANConfig",
     "BSBLANAuthError",
+    "BSBLANConfig",
     "BSBLANConnectionError",
     "BSBLANError",
     "DHWTimeSwitchPrograms",
-    "Info",
-    "State",
     "Device",
-    "Sensor",
-    "StaticState",
+    "DeviceTime",
+    "HotWaterConfig",
+    "HotWaterSchedule",
     "HotWaterState",
+    "Info",
+    "Sensor",
+    "SetHotWaterParam",
+    "State",
+    "StaticState",
 ]
