@@ -4,9 +4,8 @@ from __future__ import annotations
 
 import asyncio
 import logging
-import re
 from dataclasses import dataclass, field
-from typing import TYPE_CHECKING, Any, Literal, cast
+from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast
 
 if TYPE_CHECKING:
     from collections.abc import Mapping
@@ -22,18 +21,19 @@ from .constants import (
     API_VALIDATOR_NOT_INITIALIZED_ERROR_MSG,
     API_VERSION_ERROR_MSG,
     API_VERSIONS,
+    DHW_TIME_PROGRAM_PARAMS,
     FIRMWARE_VERSION_ERROR_MSG,
     HOT_WATER_CONFIG_PARAMS,
     HOT_WATER_ESSENTIAL_PARAMS,
     HOT_WATER_SCHEDULE_PARAMS,
-    HVAC_MODE_DICT,
-    HVAC_MODE_DICT_REVERSE,
     MAX_VALID_YEAR,
     MIN_VALID_YEAR,
     MULTI_PARAMETER_ERROR_MSG,
     NO_STATE_ERROR_MSG,
     SESSION_NOT_INITIALIZED_ERROR_MSG,
+    SETTABLE_HOT_WATER_PARAMS,
     TEMPERATURE_RANGE_ERROR_MSG,
+    VALID_HVAC_MODES,
     VERSION_ERROR_MSG,
     APIConfig,
 )
@@ -47,16 +47,16 @@ from .exceptions import (
 from .models import (
     Device,
     DeviceTime,
-    DHWTimeSwitchPrograms,
     HotWaterConfig,
     HotWaterSchedule,
     HotWaterState,
     Info,
     Sensor,
+    SetHotWaterParam,
     State,
     StaticState,
 )
-from .utility import APIValidator
+from .utility import APIValidator, validate_time_format
 
 if TYPE_CHECKING:
     from typing import Self
@@ -65,7 +65,14 @@ if TYPE_CHECKING:
 
 SectionLiteral = Literal["heating", "staticValues", "device", "sensor", "hot_water"]
 
-logging.basicConfig(level=logging.DEBUG)
+# TypeVar for hot water data models
+HotWaterDataT = TypeVar(
+    "HotWaterDataT", HotWaterState, HotWaterConfig, HotWaterSchedule
+)
+
+# TypeVar for section data models
+SectionDataT = TypeVar("SectionDataT", State, Sensor, StaticState, Info)
+
 logger = logging.getLogger(__name__)
 
 
@@ -138,7 +145,7 @@ class BSBLAN:
 
         # Initialize API data if not already done
         if self._api_data is None:
-            self._api_data = API_VERSIONS[self._api_version]
+            self._api_data = self._copy_api_config()
 
         # Initialize the API validator
         self._api_validator = APIValidator(self._api_data)
@@ -152,14 +159,25 @@ class BSBLAN:
             "hot_water",
         ]
         for section in sections:
-            await self._validate_api_section(section)
+            response_data = await self._validate_api_section(section)
 
-    async def _validate_api_section(self, section: SectionLiteral) -> None:
+            # Extract temperature unit from heating section validation
+            # (parameter 710 - target_temperature is always in heating section)
+            if section == "heating" and response_data:
+                self._extract_temperature_unit_from_response(response_data)
+
+    async def _validate_api_section(
+        self, section: SectionLiteral
+    ) -> dict[str, Any] | None:
         """Validate a specific section of the API configuration.
 
         Args:
             section: The section name to validate
 
+        Returns:
+            dict[str, Any] | None: The response data from the device, or None if
+                section was already validated or validation failed
+
         Raises:
             BSBLANError: If the API validator is not initialized
 
@@ -174,7 +192,7 @@ class BSBLAN:
         api_validator = self._api_validator
 
         if api_validator.is_section_validated(section):
-            return
+            return None
 
         # Get parameters for the section
         try:
@@ -204,6 +222,8 @@ class BSBLAN:
             # Reset validation state for this section
             api_validator.reset_validation(section)
             raise
+        else:
+            return response_data
 
     def _populate_hot_water_cache(self) -> None:
         """Populate the hot water parameter cache with all available parameters."""
@@ -215,6 +235,43 @@ class BSBLAN:
         self._hot_water_param_cache = hotwater_params.copy()
         logger.debug("Cached %d hot water parameters", len(self._hot_water_param_cache))
 
+    def _extract_temperature_unit_from_response(
+        self, response_data: dict[str, Any]
+    ) -> None:
+        """Extract temperature unit from heating section response data.
+
+        Gets the unit from parameter 710 (target_temperature) which is always
+        present in the heating section.
+
+        Args:
+            response_data: The response data from heating section validation
+
+        """
+        # Look for parameter 710 (target_temperature) in the response
+        for param_id, param_data in response_data.items():
+            # Check if this is parameter 710 and has unit information
+            if param_id == "710" and isinstance(param_data, dict):
+                unit = param_data.get("unit", "")
+                if unit in ("&deg;C", "°C"):
+                    self._temperature_unit = "°C"
+                elif unit == "°F":
+                    self._temperature_unit = "°F"
+                else:
+                    # Keep default if unit is empty or unknown
+                    logger.debug(
+                        "Unknown or empty temperature unit from parameter 710: '%s'. "
+                        "Using default (°C)",
+                        unit,
+                    )
+                logger.debug("Temperature unit set to: %s", self._temperature_unit)
+                return
+
+        # If we didn't find parameter 710, log a warning
+        logger.warning(
+            "Could not find parameter 710 in heating section response. "
+            "Using default temperature unit (°C)"
+        )
+
     def set_hot_water_cache(self, params: dict[str, str]) -> None:
         """Set the hot water parameter cache manually (for testing).
 
@@ -256,23 +313,40 @@ class BSBLAN:
             raise BSBLANVersionError(VERSION_ERROR_MSG)
 
     async def _initialize_temperature_range(self) -> None:
-        """Initialize the temperature range from static values."""
+        """Initialize the temperature range from static values.
+
+        Note: Temperature unit is extracted during API validator initialization
+        from the heating section response (parameter 710), so no extra API call
+        is needed here.
+        """
         if not self._temperature_range_initialized:
-            static_values = await self.static_values()
-            self._min_temp = float(static_values.min_temp.value)
-            self._max_temp = float(static_values.max_temp.value)
+            # Try to get temperature range from static values
+            try:
+                static_values = await self.static_values()
+                if static_values.min_temp is not None:
+                    self._min_temp = float(static_values.min_temp.value)
+                    logger.debug("Min temperature initialized: %f", self._min_temp)
+                else:
+                    logger.warning(
+                        "min_temp not available from device, "
+                        "temperature range will be None"
+                    )
+
+                if static_values.max_temp is not None:
+                    self._max_temp = float(static_values.max_temp.value)
+                    logger.debug("Max temperature initialized: %f", self._max_temp)
+                else:
+                    logger.warning(
+                        "max_temp not available from device, "
+                        "temperature range will be None"
+                    )
+            except BSBLANError as err:
+                logger.warning(
+                    "Failed to get static values: %s. Temperature range will be None",
+                    str(err),
+                )
+
             self._temperature_range_initialized = True
-            logger.debug(
-                "Temperature range initialized: min=%f, max=%f",
-                self._min_temp,
-                self._max_temp,
-            )
-            # also set unit of temperature
-            if static_values.min_temp.unit in ("&deg;C", "°C"):
-                self._temperature_unit = "°C"
-            else:
-                self._temperature_unit = "°F"
-            logger.debug("Temperature unit: %s", self._temperature_unit)
 
     @property
     def get_temperature_unit(self) -> str:
@@ -299,14 +373,33 @@ class BSBLAN:
 
         """
         if self._api_data is None:
-            if self._api_version is None:
-                raise BSBLANError(API_VERSION_ERROR_MSG)
-            self._api_data = API_VERSIONS[self._api_version]
+            self._api_data = self._copy_api_config()
             logger.debug("API data initialized for version: %s", self._api_version)
         if self._api_data is None:
             raise BSBLANError(API_DATA_NOT_INITIALIZED_ERROR_MSG)
         return self._api_data
 
+    def _copy_api_config(self) -> APIConfig:
+        """Create a copy of the API configuration for the current version.
+
+        Returns:
+            APIConfig: A deep copy of the API configuration.
+
+        Raises:
+            BSBLANError: If the API version is not set.
+
+        """
+        if self._api_version is None:
+            raise BSBLANError(API_VERSION_ERROR_MSG)
+        source_config: APIConfig = API_VERSIONS[self._api_version]
+        return cast(
+            "APIConfig",
+            {
+                section: cast("dict[str, str]", params).copy()
+                for section, params in source_config.items()
+            },
+        )
+
     async def _request(
         self,
         method: str = METH_POST,
@@ -358,8 +451,10 @@ class BSBLAN:
             raise BSBLANConnectionError(BSBLANConnectionError.message_error) from e
         except aiohttp.ClientError as e:
             raise BSBLANConnectionError(BSBLANConnectionError.message_error) from e
-        except ValueError as e:
-            raise BSBLANError(str(e)) from e
+        except (ValueError, UnicodeDecodeError) as e:
+            # Handle JSON decode errors and other parsing issues
+            error_msg = f"Invalid response format from BSB-LAN device: {e!s}"
+            raise BSBLANError(error_msg) from e
 
     def _process_response(
         self, response_data: dict[str, Any], base_path: str
@@ -463,21 +558,43 @@ class BSBLAN:
         string_params = ",".join(map(str, params))
         return {"string_par": string_params, "list": list(params.values())}
 
+    async def _fetch_section_data(
+        self,
+        section: SectionLiteral,
+        model_class: type[SectionDataT],
+    ) -> SectionDataT:
+        """Fetch data for a specific API section.
+
+        This is a generic helper method that fetches parameters for a given
+        section and returns the appropriate model.
+
+        Args:
+            section: The API section name to fetch data from.
+            model_class: The dataclass type to deserialize the response into.
+
+        Returns:
+            The populated model instance.
+
+        """
+        section_params = self._api_validator.get_section_params(section)
+        params = await self._extract_params_summary(section_params)
+        data = await self._request(params={"Parameter": params["string_par"]})
+        data = dict(zip(params["list"], list(data.values()), strict=True))
+        return model_class.from_dict(data)
+
     async def state(self) -> State:
         """Get the current state from BSBLAN device.
 
         Returns:
             State: The current state of the BSBLAN device.
 
+        Note:
+            The hvac_mode.value is returned as a raw integer from the device.
+            Use HVAC_MODE_DICT from bsblan.constants to convert to string
+            representation (e.g., "off", "auto", "eco", "heat").
+
         """
-        # Get validated parameters for heating section
-        heating_params = self._api_validator.get_section_params("heating")
-        params = await self._extract_params_summary(heating_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        # we should convert this in homeassistant integration?
-        data["hvac_mode"]["value"] = HVAC_MODE_DICT[int(data["hvac_mode"]["value"])]
-        return State.from_dict(data)
+        return await self._fetch_section_data("heating", State)
 
     async def sensor(self) -> Sensor:
         """Get the sensor information from BSBLAN device.
@@ -486,11 +603,7 @@ class BSBLAN:
             Sensor: The sensor information from the BSBLAN device.
 
         """
-        sensor_params = self._api_validator.get_section_params("sensor")
-        params = await self._extract_params_summary(sensor_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return Sensor.from_dict(data)
+        return await self._fetch_section_data("sensor", Sensor)
 
     async def static_values(self) -> StaticState:
         """Get the static information from BSBLAN device.
@@ -499,11 +612,7 @@ class BSBLAN:
             StaticState: The static information from the BSBLAN device.
 
         """
-        static_params = self._api_validator.get_section_params("staticValues")
-        params = await self._extract_params_summary(static_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return StaticState.from_dict(data)
+        return await self._fetch_section_data("staticValues", StaticState)
 
     async def device(self) -> Device:
         """Get BSBLAN device info.
@@ -522,11 +631,7 @@ class BSBLAN:
             Info: The information about the current heating system config.
 
         """
-        api_data = await self._initialize_api_data()
-        params = await self._extract_params_summary(api_data["device"])
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return Info.from_dict(data)
+        return await self._fetch_section_data("device", Info)
 
     async def time(self) -> DeviceTime:
         """Get the current time from the BSB-LAN device.
@@ -564,13 +669,16 @@ class BSBLAN:
     async def thermostat(
         self,
         target_temperature: str | None = None,
-        hvac_mode: str | None = None,
+        hvac_mode: int | None = None,
     ) -> None:
         """Change the state of the thermostat through BSB-Lan.
 
         Args:
             target_temperature (str | None): The target temperature to set.
-            hvac_mode (str | None): The HVAC mode to set.
+            hvac_mode (int | None): The HVAC mode to set as raw integer value.
+                Valid values: 0=off, 1=auto, 2=eco, 3=heat.
+                Use HVAC_MODE_DICT_REVERSE from bsblan.constants to convert
+                string names to integers.
 
         """
         await self._initialize_temperature_range()
@@ -582,18 +690,18 @@ class BSBLAN:
         )
 
         state = self._prepare_thermostat_state(target_temperature, hvac_mode)
-        await self._set_thermostat_state(state)
+        await self._set_device_state(state)
 
     def _prepare_thermostat_state(
         self,
         target_temperature: str | None,
-        hvac_mode: str | None,
+        hvac_mode: int | None,
     ) -> dict[str, Any]:
         """Prepare the thermostat state for setting.
 
         Args:
             target_temperature (str | None): The target temperature to set.
-            hvac_mode (str | None): The HVAC mode to set.
+            hvac_mode (int | None): The HVAC mode to set as raw integer.
 
         Returns:
             dict[str, Any]: The prepared state for the thermostat.
@@ -610,7 +718,7 @@ class BSBLAN:
             state.update(
                 {
                     "Parameter": "700",
-                    "EnumValue": HVAC_MODE_DICT_REVERSE[hvac_mode],
+                    "Value": str(hvac_mode),
                     "Type": "1",
                 },
             )
@@ -637,18 +745,18 @@ class BSBLAN:
         except ValueError as err:
             raise BSBLANInvalidParameterError(target_temperature) from err
 
-    def _validate_hvac_mode(self, hvac_mode: str) -> None:
+    def _validate_hvac_mode(self, hvac_mode: int) -> None:
         """Validate the HVAC mode.
 
         Args:
-            hvac_mode (str): The HVAC mode to validate.
+            hvac_mode (int): The HVAC mode to validate (0-3).
 
         Raises:
             BSBLANInvalidParameterError: If the HVAC mode is invalid.
 
         """
-        if hvac_mode not in HVAC_MODE_DICT_REVERSE:
-            raise BSBLANInvalidParameterError(hvac_mode)
+        if hvac_mode not in VALID_HVAC_MODES:
+            raise BSBLANInvalidParameterError(str(hvac_mode))
 
     def _validate_time_format(self, time_value: str) -> None:
         """Validate the time format.
@@ -660,58 +768,64 @@ class BSBLAN:
             BSBLANInvalidParameterError: If the time format is invalid.
 
         """
-        # BSB-LAN supports format: DD.MM.YYYY HH:MM:SS (e.g., "13.08.2025 10:25:55")
-        pattern = r"^(\d{2})\.(\d{2})\.(\d{4}) (\d{2}):(\d{2}):(\d{2})$"
+        try:
+            validate_time_format(time_value, MIN_VALID_YEAR, MAX_VALID_YEAR)
+        except ValueError as err:
+            raise BSBLANInvalidParameterError(str(err)) from err
 
-        match = re.match(pattern, time_value)
-        if not match:
-            msg = f"Invalid time format: {time_value}. Expected DD.MM.YYYY HH:MM:SS"
-            raise BSBLANInvalidParameterError(msg)
+    async def _set_device_state(self, state: dict[str, Any]) -> None:
+        """Set device state via BSB-LAN API.
 
-        day, month, year, hour, minute, second = map(int, match.groups())
-
-        # Validate ranges
-        if not (1 <= day <= 31):
-            msg = f"Invalid day: {day}"
-            raise BSBLANInvalidParameterError(msg)
-        if not (1 <= month <= 12):
-            msg = f"Invalid month: {month}"
-            raise BSBLANInvalidParameterError(msg)
-        if not (MIN_VALID_YEAR <= year <= MAX_VALID_YEAR):
-            msg = f"Invalid year: {year}"
-            raise BSBLANInvalidParameterError(msg)
-        if not (0 <= hour <= 23):
-            msg = f"Invalid hour: {hour}"
-            raise BSBLANInvalidParameterError(msg)
-        if not (0 <= minute <= 59):
-            msg = f"Invalid minute: {minute}"
-            raise BSBLANInvalidParameterError(msg)
-        if not (0 <= second <= 59):
-            msg = f"Invalid second: {second}"
-            raise BSBLANInvalidParameterError(msg)
-
-        # Additional validation for days per month
-        if month in [4, 6, 9, 11] and day > 30:
-            msg = f"Invalid day {day} for month {month}"
-            raise BSBLANInvalidParameterError(msg)
-        if month == 2:
-            # Leap year check
-            is_leap = (year % 4 == 0 and year % 100 != 0) or (year % 400 == 0)
-            max_day = 29 if is_leap else 28
-            if day > max_day:
-                msg = f"Invalid day {day} for February in year {year}"
-                raise BSBLANInvalidParameterError(msg)
-
-    async def _set_thermostat_state(self, state: dict[str, Any]) -> None:
-        """Set the thermostat state.
+        This is a unified method for setting thermostat and hot water state.
 
         Args:
-            state (dict[str, Any]): The state to set for the thermostat.
+            state (dict[str, Any]): The state to set on the device.
 
         """
         response = await self._request(base_path="/JS", data=state)
         logger.debug("Response for setting: %s", response)
 
+    async def _fetch_hot_water_data(
+        self,
+        param_filter: set[str],
+        model_class: type[HotWaterDataT],
+        error_msg: str,
+    ) -> HotWaterDataT:
+        """Fetch hot water data for a specific parameter set.
+
+        This is a generic helper method that fetches hot water parameters
+        based on the provided filter and returns the appropriate model.
+
+        Args:
+            param_filter: Set of parameter IDs to fetch.
+            model_class: The dataclass type to deserialize the response into.
+            error_msg: Error message if no parameters are available.
+
+        Returns:
+            The populated model instance.
+
+        Raises:
+            BSBLANError: If no parameters are available for the filter.
+
+        """
+        hotwater_params = (
+            self._hot_water_param_cache
+            or self._api_validator.get_section_params("hot_water")
+        )
+        filtered_params = {
+            param_id: param_name
+            for param_id, param_name in hotwater_params.items()
+            if param_id in param_filter
+        }
+
+        if not filtered_params:
+            raise BSBLANError(error_msg)
+
+        params = await self._extract_params_summary(filtered_params)
+        data = await self._request(params={"Parameter": params["string_par"]})
+        data = dict(zip(params["list"], list(data.values()), strict=True))
+        return model_class.from_dict(data)
+
     async def hot_water_state(self) -> HotWaterState:
         """Get essential hot water state for frequent polling.
 
@@ -723,25 +837,11 @@ class BSBLAN:
             HotWaterState: Essential hot water state information.
 
         """
-        # Use cached parameters or fall back to API validator
-        hotwater_params = (
-            self._hot_water_param_cache
-            or self._api_validator.get_section_params("hot_water")
+        return await self._fetch_hot_water_data(
+            param_filter=HOT_WATER_ESSENTIAL_PARAMS,
+            model_class=HotWaterState,
+            error_msg="No essential hot water parameters available",
         )
-        essential_params = {
-            param_id: param_name
-            for param_id, param_name in hotwater_params.items()
-            if param_id in HOT_WATER_ESSENTIAL_PARAMS
-        }
-
-        if not essential_params:
-            msg = "No essential hot water parameters available"
-            raise BSBLANError(msg)
-
-        params = await self._extract_params_summary(essential_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return HotWaterState.from_dict(data)
 
     async def hot_water_config(self) -> HotWaterConfig:
         """Get hot water configuration and advanced settings.
@@ -753,25 +853,11 @@ class BSBLAN:
             HotWaterConfig: Hot water configuration information.
 
         """
-        # Use cached parameters or fall back to API validator
-        hotwater_params = (
-            self._hot_water_param_cache
-            or self._api_validator.get_section_params("hot_water")
+        return await self._fetch_hot_water_data(
+            param_filter=HOT_WATER_CONFIG_PARAMS,
+            model_class=HotWaterConfig,
+            error_msg="No hot water configuration parameters available",
         )
-        config_params = {
-            param_id: param_name
-            for param_id, param_name in hotwater_params.items()
-            if param_id in HOT_WATER_CONFIG_PARAMS
-        }
-
-        if not config_params:
-            msg = "No hot water configuration parameters available"
-            raise BSBLANError(msg)
-
-        params = await self._extract_params_summary(config_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return HotWaterConfig.from_dict(data)
 
     async def hot_water_schedule(self) -> HotWaterSchedule:
         """Get hot water time program schedules.
@@ -783,148 +869,75 @@ class BSBLAN:
             HotWaterSchedule: Hot water schedule information.
 
         """
-        # Use cached parameters or fall back to API validator
-        hotwater_params = (
-            self._hot_water_param_cache
-            or self._api_validator.get_section_params("hot_water")
+        return await self._fetch_hot_water_data(
+            param_filter=HOT_WATER_SCHEDULE_PARAMS,
+            model_class=HotWaterSchedule,
+            error_msg="No hot water schedule parameters available",
         )
-        schedule_params = {
-            param_id: param_name
-            for param_id, param_name in hotwater_params.items()
-            if param_id in HOT_WATER_SCHEDULE_PARAMS
-        }
 
-        if not schedule_params:
-            msg = "No hot water schedule parameters available"
-            raise BSBLANError(msg)
-
-        params = await self._extract_params_summary(schedule_params)
-        data = await self._request(params={"Parameter": params["string_par"]})
-        data = dict(zip(params["list"], list(data.values()), strict=True))
-        return HotWaterSchedule.from_dict(data)
-
-    async def set_hot_water(  # noqa: PLR0913
-        self,
-        nominal_setpoint: float | None = None,
-        reduced_setpoint: float | None = None,
-        operating_mode: str | None = None,
-        dhw_time_programs: DHWTimeSwitchPrograms | None = None,
-        eco_mode_selection: str | None = None,
-        dhw_charging_priority: str | None = None,
-        legionella_dwelling_time: float | None = None,
-        legionella_circulation_pump: str | None = None,
-        legionella_circulation_temp_diff: float | None = None,
-        dhw_circulation_pump_release: str | None = None,
-        dhw_circulation_pump_cycling: float | None = None,
-        dhw_circulation_setpoint: float | None = None,
-        operating_mode_changeover: str | None = None,
-    ) -> None:  # pylint: disable=too-many-arguments,too-many-positional-arguments,too-many-locals
+    async def set_hot_water(self, params: SetHotWaterParam) -> None:
         """Change the state of the hot water system through BSB-Lan.
 
+        Only one parameter should be set at a time (BSB-LAN API limitation).
+
+        Example:
+            params = SetHotWaterParam(nominal_setpoint=55.0)
+            await client.set_hot_water(params)
+
         Args:
-            nominal_setpoint (float | None): The nominal setpoint temperature to set.
-            reduced_setpoint (float | None): The reduced setpoint temperature to set.
-            operating_mode (str | None): The operating mode to set.
-            dhw_time_programs (DHWTimeSwitchPrograms | None): Time switch programs.
-            eco_mode_selection (str | None): Eco mode selection.
-            dhw_charging_priority (str | None): DHW charging priority.
-            legionella_dwelling_time (float | None): Legionella dwelling time.
-            legionella_circulation_pump (str | None): Legionella circulation pump.
-            legionella_circulation_temp_diff (float | None): Legionella circulation
-                temperature difference.
-            dhw_circulation_pump_release (str | None): DHW circulation pump release.
-            dhw_circulation_pump_cycling (float | None): DHW circulation pump cycling.
-            dhw_circulation_setpoint (float | None): DHW circulation setpoint.
-            operating_mode_changeover (str | None): Operating mode changeover.
+            params: SetHotWaterParam object containing the parameter to set.
+
+        Raises:
+            BSBLANError: If multiple parameters are set or no parameter is set.
 
         """
         # Validate only one parameter is being set
-        time_program_params = []
-        if dhw_time_programs:
-            if dhw_time_programs.monday:
-                time_program_params.append(dhw_time_programs.monday)
-            if dhw_time_programs.tuesday:
-                time_program_params.append(dhw_time_programs.tuesday)
-            if dhw_time_programs.wednesday:
-                time_program_params.append(dhw_time_programs.wednesday)
-            if dhw_time_programs.thursday:
-                time_program_params.append(dhw_time_programs.thursday)
-            if dhw_time_programs.friday:
-                time_program_params.append(dhw_time_programs.friday)
-            if dhw_time_programs.saturday:
-                time_program_params.append(dhw_time_programs.saturday)
-            if dhw_time_programs.sunday:
-                time_program_params.append(dhw_time_programs.sunday)
-            if dhw_time_programs.standard_values:
-                time_program_params.append(dhw_time_programs.standard_values)
+        time_program_params: list[str] = []
+        if params.dhw_time_programs:
+            programs = params.dhw_time_programs
+            time_program_params.extend(
+                prog
+                for prog in [
+                    programs.monday,
+                    programs.tuesday,
+                    programs.wednesday,
+                    programs.thursday,
+                    programs.friday,
+                    programs.saturday,
+                    programs.sunday,
+                    programs.standard_values,
+                ]
+                if prog
+            )
 
         self._validate_single_parameter(
-            nominal_setpoint,
-            reduced_setpoint,
-            operating_mode,
-            eco_mode_selection,
-            dhw_charging_priority,
-            legionella_dwelling_time,
-            legionella_circulation_pump,
-            legionella_circulation_temp_diff,
-            dhw_circulation_pump_release,
-            dhw_circulation_pump_cycling,
-            dhw_circulation_setpoint,
-            operating_mode_changeover,
+            params.nominal_setpoint,
+            params.reduced_setpoint,
+            params.nominal_setpoint_max,
+            params.operating_mode,
+            params.eco_mode_selection,
+            params.dhw_charging_priority,
+            params.legionella_function_setpoint,
+            params.legionella_function_periodicity,
+            params.legionella_function_day,
+            params.legionella_function_time,
+            params.legionella_function_dwelling_time,
+            params.operating_mode_changeover,
             *time_program_params,
             error_msg=MULTI_PARAMETER_ERROR_MSG,
         )
 
-        state = self._prepare_hot_water_state(
-            nominal_setpoint,
-            reduced_setpoint,
-            operating_mode,
-            dhw_time_programs,
-            eco_mode_selection,
-            dhw_charging_priority,
-            legionella_dwelling_time,
-            legionella_circulation_pump,
-            legionella_circulation_temp_diff,
-            dhw_circulation_pump_release,
-            dhw_circulation_pump_cycling,
-            dhw_circulation_setpoint,
-            operating_mode_changeover,
-        )
-        await self._set_hot_water_state(state)
+        state = self._prepare_hot_water_state(params)
+        await self._set_device_state(state)
 
-    def _prepare_hot_water_state(  # noqa: PLR0913, PLR0912
+    def _prepare_hot_water_state(
         self,
-        nominal_setpoint: float | None,
-        reduced_setpoint: float | None,
-        operating_mode: str | None,
-        dhw_time_programs: DHWTimeSwitchPrograms | None = None,
-        eco_mode_selection: str | None = None,
-        dhw_charging_priority: str | None = None,
-        legionella_dwelling_time: float | None = None,
-        legionella_circulation_pump: str | None = None,
-        legionella_circulation_temp_diff: float | None = None,
-        dhw_circulation_pump_release: str | None = None,
-        dhw_circulation_pump_cycling: float | None = None,
-        dhw_circulation_setpoint: float | None = None,
-        operating_mode_changeover: str | None = None,
-    ) -> dict[str, Any]:  # pylint: disable=too-many-arguments,too-many-positional-arguments,too-many-locals,too-many-branches
+        params: SetHotWaterParam,
+    ) -> dict[str, Any]:
         """Prepare the hot water state for setting.
 
         Args:
-            nominal_setpoint (float | None): The nominal setpoint temperature to set.
-            reduced_setpoint (float | None): The reduced setpoint temperature to set.
-            operating_mode (str | None): The operating mode to set.
-            dhw_time_programs (DHWTimeSwitchPrograms | None): Time switch programs.
-            eco_mode_selection (str | None): Eco mode selection.
-            dhw_charging_priority (str | None): DHW charging priority.
-            legionella_dwelling_time (float | None): Legionella dwelling time.
-            legionella_circulation_pump (str | None): Legionella circulation pump.
-            legionella_circulation_temp_diff (float | None): Legionella circulation
-                temperature difference.
-            dhw_circulation_pump_release (str | None): DHW circulation pump release.
-            dhw_circulation_pump_cycling (float | None): DHW circulation pump cycling.
-            dhw_circulation_setpoint (float | None): DHW circulation setpoint.
-            operating_mode_changeover (str | None): Operating mode changeover.
+            params: SetHotWaterParam object containing the parameter to set.
 
         Returns:
             dict[str, Any]: The prepared state for the hot water.
@@ -934,121 +947,20 @@ class BSBLAN:
 
         """
         state: dict[str, Any] = {}
-        if nominal_setpoint is not None:
-            state.update(
-                {"Parameter": "1610", "Value": str(nominal_setpoint), "Type": "1"},
-            )
-        if reduced_setpoint is not None:
-            state.update(
-                {"Parameter": "1612", "Value": str(reduced_setpoint), "Type": "1"},
-            )
-        if operating_mode is not None:
-            state.update(
-                {
-                    "Parameter": "1600",
-                    "EnumValue": operating_mode,
-                    "Type": "1",
-                },
-            )
-        if eco_mode_selection is not None:
-            state.update(
-                {
-                    "Parameter": "1601",
-                    "EnumValue": eco_mode_selection,
-                    "Type": "1",
-                },
-            )
-        if dhw_charging_priority is not None:
-            state.update(
-                {
-                    "Parameter": "1630",
-                    "EnumValue": dhw_charging_priority,
-                    "Type": "1",
-                },
-            )
-        if legionella_dwelling_time is not None:
-            state.update(
-                {
-                    "Parameter": "1646",
-                    "Value": str(legionella_dwelling_time),
-                    "Type": "1",
-                },
-            )
-        if legionella_circulation_pump is not None:
-            state.update(
-                {
-                    "Parameter": "1647",
-                    "EnumValue": legionella_circulation_pump,
-                    "Type": "1",
-                },
-            )
-        if legionella_circulation_temp_diff is not None:
-            state.update(
-                {
-                    "Parameter": "1648",
-                    "Value": str(legionella_circulation_temp_diff),
-                    "Type": "1",
-                },
-            )
-        if dhw_circulation_pump_release is not None:
-            state.update(
-                {
-                    "Parameter": "1660",
-                    "EnumValue": dhw_circulation_pump_release,
-                    "Type": "1",
-                },
-            )
-        if dhw_circulation_pump_cycling is not None:
-            state.update(
-                {
-                    "Parameter": "1661",
-                    "Value": str(dhw_circulation_pump_cycling),
-                    "Type": "1",
-                },
-            )
-        if dhw_circulation_setpoint is not None:
-            state.update(
-                {
-                    "Parameter": "1663",
-                    "Value": str(dhw_circulation_setpoint),
-                    "Type": "1",
-                },
-            )
-        if operating_mode_changeover is not None:
-            state.update(
-                {
-                    "Parameter": "1680",
-                    "EnumValue": operating_mode_changeover,
-                    "Type": "1",
-                },
-            )
 
-        if dhw_time_programs:
-            time_program_mapping = {
-                "561": dhw_time_programs.monday,
-                "562": dhw_time_programs.tuesday,
-                "563": dhw_time_programs.wednesday,
-                "564": dhw_time_programs.thursday,
-                "565": dhw_time_programs.friday,
-                "566": dhw_time_programs.saturday,
-                "567": dhw_time_programs.sunday,
-                "576": dhw_time_programs.standard_values,
-            }
+        # Process all mapped parameters using constants
+        for param_id, attr_name in SETTABLE_HOT_WATER_PARAMS.items():
+            value = getattr(params, attr_name)
+            if value is not None:
+                state.update({"Parameter": param_id, "Value": str(value), "Type": "1"})
 
-            for param, value in time_program_mapping.items():
+        # Process time programs if provided using constants
+        if params.dhw_time_programs:
+            for param_id, attr_name in DHW_TIME_PROGRAM_PARAMS.items():
+                value = getattr(params.dhw_time_programs, attr_name)
                 if value is not None:
-                    state.update({"Parameter": param, "Value": value, "Type": "1"})
+                    state.update({"Parameter": param_id, "Value": value, "Type": "1"})
 
         if not state:
             raise BSBLANError(NO_STATE_ERROR_MSG)
         return state
-
-    async def _set_hot_water_state(self, state: dict[str, Any]) -> None:
-        """Set the hot water state.
-
-        Args:
-            state (dict[str, Any]): The state to set for the hot water.
-
-        """
-        response = await self._request(base_path="/JS", data=state)
-        logger.debug("Response for setting: %s", response)
